/*------------------------------------------------------------------------
 Module:        $File: F:\TinyCrack\Source\tools\merge_passwd.c$
 Author:        Ralph Roth
 Project:       TinyCrack
 State:         $Id: merge_passwd.c,v 1.9 2004/08/20 07:13:52 ralproth Exp $
 Description:   Merges, sorts and weeds UNIX password files
 History:
                $Log: merge_passwd.c,v $
                Revision 1.9  2004/08/20 07:13:52  ralproth
                more fixes for OpenBSD salts and hashes

                Revision 1.8  2003/01/07 05:23:47  ralph
                ! Fixes for OpenBSD Blowfish crypt()

                Revision 1.7  2003/01/03 15:39:05  ralph
                ! Makefile enhanced
                ! y2k3 + copyrigth
                ! Samba/LM fixes

                Revision 1.6  2002/04/15 20:35:52  ralph
                Added password_filter.lex
                Misc spelling errors fixed

                Revision 1.5  2002/04/11 16:17:12  ralph
                Dos2ux applied

                Revision 1.4  2002/01/03 12:59:00  ralph
                Fixed various bugs found in MergePasswords and other tools

                Revision 1.3  2001/10/27 22:15:37  ralph
                DOS2UX

------------------------------------------------------------------------*/
static char *_CMSID = "$CmsFile: merge_passwd $ $Date: 2004/08/20 07:13:52 $";

/* Password File Merger, merges password files from cmd line, rar 29-08-2000 */
/* 11-Apr-2000 tried to compiled it under GNU/MinWG                          */
/* 03-Sep-2001 ported to BCC32/5.5                                           */
/* 27-Dec-2001 ported to TC++ 3.00                                           */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined (__TURBOC__) || (MSDOS)
#define MAXVALHASH 0x1fff       /* TC -> 7fff */
char *  strsep(char **_stringp, const char *_delim);
char *  index(const char *_string, int _c);
#else
#define MAXVALHASH 0xffff       /* gcc, 32bit etc. */
#endif

extern char * optarg;
extern int    optind;


#define TRUE    (1 == 1)
#define FALSE   (!TRUE)
#define PASSWL  "all_lists.pw"
#define BADPASS "bad_pass.list"

struct salt_type {
    char user[30], salt[80];		/* OpenBSD Blowfish = 60 chars! rar 06.01.03 */
    long id, gid;
    char info[100], home[80], shell[80];

    struct salt_type *next;
};

struct salt_type *current, *salt_hash[MAXVALHASH];

long    Cnt = 0;
int     bVerbose = FALSE;
char    szOutFile[256], szBadFile[256];

int salt_hashfunc(struct salt_type *s)
{

    int  h;

    if (strlen(s->salt) < 8)
       return 0;

    h = s->salt[0];
    h <<= 4;
    h = h + s->salt[4];
    return h &= MAXVALHASH;
}

int exists_salt_hash(struct salt_type *sin)
{
    current = salt_hash[salt_hashfunc(sin)];

    while (current != NULL)
    {
        if (strcmp(sin->salt, current->salt) == 0)
        {
            return -1;
        }
        current = current->next;
    }

    return 0;

} /* exists_salt_hash() */


void ins_salt_hash(struct salt_type *s)
{

    int nsalt_hash = salt_hashfunc(s);

    if (exists_salt_hash(s) )
        return;

    current = salt_hash[nsalt_hash];

    if ((salt_hash[nsalt_hash]=malloc(sizeof(struct salt_type)))==NULL)
    {
        puts("\nFATAL: Out of memory! Aborting!");
        exit(2);
    }

    memcpy(salt_hash[nsalt_hash], s, sizeof(struct salt_type));
    salt_hash[nsalt_hash]->next = current;

    Cnt++;

} /* ins_salt_hash() */


int read_passwordfile(char *szSrc)
{
    FILE *hIn, *hOut;
    char s[256], *sp, s3[3], szOriginal[256];
    char *p, gid[30], id[30];
    long lCountOld = Cnt;
    struct salt_type  sTmp;
    int c1;

    if ((hIn=fopen(szSrc,"r")) == NULL)
    {
        perror(szSrc);
        return -1;
    }
    if ((hOut=fopen(szBadFile,"at")) == NULL)
    {
        perror("Bad password file (-b badpass!.pw)");
        return -1;
    }

    while (!feof(hIn)) {
        fgets(s, sizeof(s), hIn);
        strcpy(szOriginal, s);
        if ((p = strchr(s, '\r')) != NULL) *p = 0;
        if ((p = strchr(s, '\n')) != NULL) *p = 0;
        if (!(p = strstr(s, ":"))) continue;

        sp = s;
        strcpy(sTmp.user, (char *) strsep(&sp, ":"));
        strcpy(sTmp.salt, (char *) strsep(&sp, ":"));   /* index */
        strcpy(id, (char *) strsep(&sp, ":"));
        strcpy(gid, (char *) strsep(&sp, ":"));
        strcpy(sTmp.info,  (char *)strsep(&sp, ":"));
        strcpy(sTmp.home,  (char *)strsep(&sp, ":"));
        strcpy(sTmp.shell,  (char *)strsep(&sp, ":\n"));

        sTmp.id = atol(id);
        sTmp.gid = atol(gid);

        strncpy(s3, sTmp.salt, 2);
        c1 = s3[0];
        if ((index("!:*", c1) != NULL) && \
           (s3[0] == s3[1]))
           strcpy(sTmp.salt, (char*) &sTmp.salt[2]);

        if ((sTmp.salt[0] == '*') && (strlen(sTmp.salt) == 14))
           strcpy(sTmp.salt, &sTmp.salt[1]);

        c1 = sTmp.salt[0];      /* locked passwd? */

        if ((strlen(sTmp.salt) != 1) && (!exists_salt_hash(&sTmp)) && \
            (index("!-?*", c1)==NULL))
                         ins_salt_hash(&sTmp);
        else
            fputs(szOriginal, hOut);
    }
    fclose(hIn);
    fclose(hOut);

    printf("%-14s %7ld/%ld new passwords added. (%ld bytes used)\n", \
           szSrc, Cnt-lCountOld, Cnt, (sizeof(salt_hash)+Cnt*sizeof(struct salt_type)));

    return 0;

}  /* read_passwordfile() */


void write_passwordfile(char *szFile)
{
    int   t;
    FILE *hOut;

    if ((hOut=fopen(szFile,"wt")) != NULL)
    {

        for (t=0; t<=MAXVALHASH; t++)		/* FIXED: <MAXVALHASH */
        {
            current = salt_hash[t];

            while (current != NULL)
            {
                if (strlen(current->user) > 0)
                   fprintf(hOut, "%s:%s:%ld:%ld:%s:%s:%s\n", \
                              current->user, current->salt, \
                              current->id, current->gid, \
                              current->info, current->home, current->shell);
                current = current->next;
            }
        }
        fclose(hOut);
    }

} /* write_passwordfile() */


/*------------------------------------------------------------------------
 Procedure:     main ID:1
 Purpose:       Main guts
 Input:
 Output:
 Errors:

------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
    char   szFile[256];
    int    nI;

    printf("*NIX password list merger (%s), inital hash buffer = %ld bytes.\n%s -? for help, (c) by ROSE SWE\n", \
            __DATE__, (long) sizeof(salt_hash), argv[0]);


    strcpy(szOutFile, PASSWL);
    strcpy(szBadFile, BADPASS);

    while((nI = getopt(argc, argv, "vV?hHo:O:b:B:")) != EOF)
    {
        switch(toupper(nI))
        {
        case 'V':
            bVerbose = TRUE;
            puts("Verbose mode = ON!");
            break;
        case '?':
        case 'H':
                printf("\nUsage: password_merge -v -?|h|H -oOutPass.PW -bBadPass.PW  inputfile(s)\n\n");
                printf("-v -V             more verbose messages, default = OFF\n");
                printf("-? -h -H          this little help\n");
                printf("-bFile            output file name for bad entries, default = %s\n", BADPASS);
                printf("-oFile            output file name for merged lists, default = %s\n", PASSWL);
            exit(-1);
            break;
        case 'O':
            strcpy(szOutFile, optarg);
            break;
        case 'B':
            strcpy(szBadFile, optarg);
            break;
        default:
            printf("ERROR: Unknown command line option '%c'!\n", nI);
            exit(-1);
            break;
        }
    } /* while */

    /* printf("argc=%i optind=%i %s\n", argc, optind, argv[optind]); */

    memset(salt_hash, 0, sizeof(salt_hash));

    while (--argc>=optind) {
          strcpy(szFile, argv[argc]);
          if (bVerbose == TRUE)
                 printf("Reading file %s\n", szFile);
          fflush(stdout);
          read_passwordfile(szFile);
    }
    printf("Writing back one cummulated password file (%s, %ld passwords)\n", szOutFile, Cnt);
    write_passwordfile(szOutFile);


    return 0;
}
