/****************************************************************************
 * $Id: merge_pots.c,v 1.14 2004/09/16 20:59:35 ralproth Exp $
 *
 * $Log: merge_pots.c,v $
 * Revision 1.14  2004/09/16 20:59:35  ralproth
 * Changes made in RW
 *
 * Revision 1.13  2004/08/20 07:13:52  ralproth
 * more fixes for OpenBSD salts and hashes
 *
 * Revision 1.12  2004/06/22 22:28:08  ralproth
 * First working version of new VBS and IRC engines
 *
 * Revision 1.11  2003/01/24 18:04:01  ralph
 * F-Merged, astyle
 *
 * Revision 1.10  2003/01/07 05:23:47  ralph
 * ! Fixes for OpenBSD Blowfish crypt()
 *
 * Revision 1.9  2003/01/03 15:39:05  ralph
 * ! Makefile enhanced
 * ! y2k3 + copyrigth
 * ! Samba/LM fixes
 *
 * Revision 1.8  2002/04/24 07:41:17  ralph
 * Added makefile
 * Changed merge_pot hashing
 *
 * Revision 1.7  2002/04/15 10:53:09  ralph
 * NT LM hashes + spell fixing
 *
 * Revision 1.6  2002/04/15 11:30:49  ralph
 * Added NT LM handling
 *
 * Revision 1.5  2002/04/11 16:17:12  ralph
 * Dos2ux applied
 *
 * Revision 1.4  2002/03/20 18:20:35  ralph
 * Enhanced: Commandline handling.
 * Fixes PGCC complains about
 * Fixes for Borland C++
 *
 * Revision 1.3  2002/01/03 12:59:00  ralph
 * Fixed various bugs found in MergePasswords and other tools
 *
 * (C)opyright by ROSE SWE, Dipl.Ing. Ralph Roth, ROSE_SWE@hotmail.com
 * All Rights Reserved!                        http://come.to/rose_swe
 *
 ****************************************************************************/



#include <stdio.h>
#include <string.h>
#include <stdlib.h>

char* 	crypt(char*, char*);
long 	Cnt = 0, lDupes = 0;

#define MAXVALHASH 0x7ff
struct 	salt_type *current, *hash[MAXVALHASH];

struct 	salt_type
{					/* OpenBSD Blowfish = 60 chars! rar 06.01.03 */
    char salt[80], passwd[64];		/* 14 -> unix crypt,  20 -> NT LM, Samba -> ???? */
    struct salt_type *next;		/* 291202, changed rar for LM passwords */
};

/* -------------------------------------------- functions --------------- */

int 	hashfunc(char *s)
{
    register int h = s[0];		/* changed from 1,0 to 0,5 (NT LM Hash) */
    h <<= 4;				/* and back, 24.04.2002 		*/
    h = h + s[1];
    return h &= MAXVALHASH;
}

int 	exists_hash(char *s, char *p)
{
    current = hash[hashfunc(s)];

    while (current != NULL)
    {
        if (strcmp(s, current->salt) == 0)
        {
            strcpy(p, current->passwd);
            return -1;
        }
        current = current->next;
    }

    return 0;

} /* exists_hash() */


void 	ins_hash(char *s, char *p)
{
    int nHash = hashfunc(s);

    if (exists_hash(s,p) )
    {   lDupes++;
        return;
    }    

    current = hash[nHash];

    if ((hash[nHash]=malloc(sizeof(struct salt_type)))==NULL)
    {
        puts("\nFATAL: Out of memory! Aborting!");
        exit(2);
    }
    strcpy(hash[nHash]->salt,s);
    strcpy(hash[nHash]->passwd,p);
    hash[nHash]->next = current;

    Cnt++;

} /* ins_hash() */


int 	read_jackpot(char *szSrc, char cDel)
{
    FILE 	*hIn;
    static char s[256];
    char     	*p;
    int      	bFMerged;
    long	lOldCount = Cnt;

    bFMerged = (strcmp(szSrc, "F-merged") == 0);
    printf("* Trying to read file: %s \n", szSrc);

    if ((hIn=fopen(szSrc,"r")) == NULL)
        return -1;

    while (!feof(hIn))
    {
        fgets(s, sizeof(s), hIn);
        if ((p = strchr(s, '\r')))
        {
            *p = 0;
        };
        if ((p = strchr(s, '\n')))
        {
            *p = 0;
        };

        if ((bFMerged) && (s[0] == 'F') && (s[1] == ':'))
            strcpy(s, &s[2]);  /* skip crack's F_Merged F:salt:password format */

        if (!(p = strchr(s, cDel)))
            continue;
        *p = 0;
        p++;

        if (!exists_hash(s,p))
            ins_hash(s, p);
    }
    fclose(hIn);

    printf("+ %s: %ld passwords read, %ld new, %ld dupes.\n", szSrc, \
            Cnt, Cnt - lOldCount, lDupes);

    return 0;

}  /* read_jackpot() */

/* -------------------------------------------------------------- */

void 	write_jackpot(char *szFile, char cDel)
{
    int   t;
    FILE *hOut;

    if ((hOut=fopen(szFile,"wt")) != NULL)
    {
        for (t=0; t<=MAXVALHASH; t++)		/* fixed, 02.01.02 */
        {
            current = hash[t];

            while (current != NULL)
            {
                fprintf(hOut, "%s%c%s\n", current->salt, cDel, current->passwd);
                current = current->next;
            }
        }
        fclose(hOut);
    }

} /* write_jackpot() */

#ifdef ENLARGEPOT

/* From local_passwd.c (C) Regents of Univ. of California blah blah */
static unsigned char itoa64[] =         /* 0 ... 63 => ascii - 64 */
        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";


void add_empty_pwds(void)
{
    int nT1, nT2;
    char *pwe;
    char pw[32];
    char salt[3];

    strcpy(pw,"trustno1");
    for (nT1=0;nT1<64; nT1++ )
    {
        for (nT2=0;nT2<64; nT2++ )
        {
            sprintf(salt, "%c%c", (char) itoa64[nT1], (char) itoa64[nT2]);
            printf("%s\r", salt);
            pwe = (char *) crypt(pw, salt);
            ins_hash(pwe, pw);
        }
    }
    printf("%ld passwords\n", Cnt);
}

#endif

int main(int argc, char **argv)
{

    printf("MergePots $Id: merge_pots.c,v 1.14 2004/09/16 20:59:35 ralproth Exp $ (%s)\n", __DATE__);
    printf("Usage: MergePorts [additional_files delimiters]  (e.g. mypot.pot|newpot.pot :)\n\n");

    memset(hash,0,sizeof(hash));

    read_jackpot("F-merged", ':');
    read_jackpot("cracked.pot", '|');
    read_jackpot("jack.pot", ':');
    read_jackpot("john.pot", ':');

    if (--argc > 1)
        while (argc > 0)
        {
            argc--;
            read_jackpot(argv[argc], argv[argc+1][0]);
            argc--;
        }

#ifdef ENLARGEPOT
    // init_des();
    puts("Enlarging jackpot with empty passwords....");
    add_empty_pwds();
#endif

    printf("* Writing back the jackpot (TinyCrack format) with %ld entries to disc....\n", Cnt);
    write_jackpot("cracked.pot", '|');
    puts("* Writing back the jackpot (JOHN format) to disc....");
    write_jackpot("john.pot", ':');

    return 0;
}
