/* $Id: passwd_filter.lex,v 1.12 2004/08/20 07:13:52 ralproth Exp $
 *
 * Fetches a valid /etc/passwd line from a file (SMB in future etc.)
 * - removes invalid/locked salts
 * - minor checks like UID and GID are performed
 * - empty passwords are ok
 *
 * $Log: passwd_filter.lex,v $
 * Revision 1.12  2004/08/20 07:13:52  ralproth
 * more fixes for OpenBSD salts and hashes
 *
 * Revision 1.11  2004/08/19 22:19:23  ralproth
 * tried some fixes for BSD and MD5 passwords
 *
 * Revision 1.10  2004/01/11 16:58:58  ralproth
 * Fixes for Debian 3.0r2//new flex
 *
 * Revision 1.9  2003/11/24 22:36:04  ralproth
 * Fixed a bug with unlink()
 *
 * Revision 1.8  2003/08/20 18:01:26  ralproth
 * Enhancements amde to compile under GCC/Dev-Cpp
 *
 * Revision 1.7  2003/07/28 22:09:09  ralproth
 * Fixed bug with password ageing
 *
 * Revision 1.6  2003/01/03 15:39:06  ralph
 * ! Makefile enhanced
 * ! y2k3 + copyrigth
 * ! Samba/LM fixes
 *
 * Revision 1.5  2002/08/20 17:13:31  ralph
 * more htpasswd fixes
 *
 * Revision 1.4  2002/08/20 16:57:14  ralph
 * Changes/fixes for htpasswd
 *
 * Revision 1.3  2002/08/20 16:41:35  ralph
 * Changes to support .htpasswd file format
 *
 * Revision 1.2  2002/04/15 12:39:52  ralph
 * Added password_filter.lex handling
 *
 * Revision 1.1  2002/04/15 12:24:40  ralph
 * Initial import into WIN CVS
 *
 */

/* Bug?
	Reading file old-cvs
	fatal flex scanner internal error--end of buffer missed
*/

%{
#include <stdio.h>
#include <string.h>
/* default yywrap function - always treat EOF as an EOF */
// #define YY_SKIP_YYWRAP 
// #define yywrap( ) 

#define BADPWL  -1
#define GOODPWL 1               /* 0 means eof by yywrap */
#define GOODSAMBA 2		/* Samba NT/LM format? */
#define LINUXMD5 3		/* BDS MD5: root:$1$sNWgx2.C$5agfsSCJW1bzS80RQWwxi1: */
#define BSDMD5	4		/* root:$2a$05$qCNdfsKeUGtPB8tjX9q2Uu0qahMNBx8vaDchbufjO/cx/TukVGHQG:12058:0:10000::::  */

static char *_passwd_filter_ID = "$Id: passwd_filter.lex,v 1.12 2004/08/20 07:13:52 ralproth Exp $";
char    szTmpyyt[1024*8];

%}
%option noyywrap

        /* white space (space/tabs) */
        /* SALT (:{ANY}{13,18}:|::) */
        /* HTTPWD (:{ANY}{13,18}) */
WS      [ \t]
DIGITS  [0-9]+
HEX     [0-9A-Fa-f\-]+
TOSF    [\.\/0-9A-Za-z\,]+
TOSFMD  [\.\/0-9A-Za-z\,\$]+
/* # { changed/added 29.07.2003 (00:00) by Ralph Roth } */
ANY     [^:]
ANYLOG  [^: */]
SALT    (:{TOSF}{13,18}:|::)
HTPWD   (:{TOSF}{13,15})
%option caseless

%%
^{ANYLOG}{2,20}{HTPWD}$         { sprintf(szTmpyyt, "%s:0:0:htpasswd:file:format:::", yytext);
                                  strcpy(yytext, szTmpyyt);
                                  return GOODPWL; };

^{ANYLOG}{2,20}{SALT}{DIGITS}:{DIGITS}:{ANY}*:{ANY}*:.*$   {
                                        /* puts(yytext);  */
                                        return GOODPWL;
                                   }; /* Samba stuff */
^{ANYLOG}{2,22}:{DIGITS}:{HEX}:{HEX}:.*$      {
					return GOODSAMBA;
					};
^{ANYLOG}{2,22}:\$1\$.{8}\${ANY}{20,28}:{HEX}:{HEX}:.*$  {    
					return LINUXMD5;
					};
^{ANYLOG}{2,22}:\$2a\${HEX}\${ANY}*:{HEX}:{HEX}:.*$      {
					return BSDMD5;
					};


{WS}|^$|^\n|^\r                     ;
.|\n                         	{ return BADPWL; /* printf("%s", yytext); */ };         /* reject */
%%

